!
! Supplementary Material, Software, for the Textbook:
!
! Mudelsee M (2020) Statistical Analysis of Climate Extremes,
!                   Cambridge University Press, 200 pp,
!                   ISBN 978-1-107-03318-4 (Hardback),
!                   ISBN 978-1-108-79146-5 (Paperback),
!                   https://www.manfredmudelsee.com/textbook
!
!=======================================================================
!
! CLsim: Cox-Lewis test with simulated series
!
!=======================================================================
!
!
! Copyright notice:     Copyright (C)
!                       Climate Risk Analysis - Manfred Mudelsee
!                       https://www.climate-risk-analysis.com
!
! Permission notice:    Permission is hereby granted, free of charge,
!                       to any person obtaining a copy of this software
!                       and associated computer files -- excluding the
!                       Numerical Recipes file(s) -- (the "Software"),
!                       to deal in the Software without restriction,
!                       including without limitation the rights to use,
!                       copy, modify, merge, publish, distribute,
!                       sublicense, and/or sell copies of the Software,
!                       and to permit persons to whom the Software is
!                       furnished to do so, subject to the following
!                       conditions:
!
!                       The above copyright notice, including the URL
!                       https://www.climate-risk-analysis.com,
!                       and this permission notice shall be included
!                       in all copies or substantial portions of the
!                       Software.
!
!=======================================================================
!
! Disclaimer:           THE SOFTWARE IS PROVIDED "AS IS", WITHOUT
!                       WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
!                       INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
!                       MERCHANTABILITY, FITNESS FOR A PARTICULAR
!                       PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
!                       THE AUTHOR OR COPYRIGHT HOLDER BE LIABLE FOR ANY
!                       CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
!                       ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
!                       FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
!                       OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
!
!=======================================================================
!
! Developer:            Climate Risk Analysis - Manfred Mudelsee
!                       Kreuzstrasse 27
!                       Heckenbeck
!                       37581 Bad Gandersheim
!                       Germany
!
! Contact:              http://www.climate-risk-analysis.com
!                       mudelsee@climate-risk-analysis.com
!
!=======================================================================
!
! Fortran 90 Compilation
! ======================
!
! gfortran compiler command: gfortran -o CLsim.exe -O2 CLsim.f90
!
! Change Log
! ==========
!
! Version 1.0       June 2019
!
! Contact
! =======
!
! Manfred Mudelsee
! Climate Risk Analysis
! Kreuzstrasse 27
! Heckenbeck
! 37581 Bad Gandersheim
! Germany
! Phone +49 5563 999 81 40
! Email mudelsee@climate-risk-analysis.com
! URL http://www.climate-risk-analysis.com
!
! ======================================================================
!
! Numerical Recipes File
! ======================
!
include 'nrtype.f90'
include 'nr.f90'
include 'nrutil.f90'
!
program CLsim
      use nrtype
      implicit none
!
      character(len=49), parameter ::                                   &
      infile = 'Hurricanes-CAT-1-5-DATA-US-landfall-1980-2018.dat'
      character(len=15), parameter ::                                   &
      outfile = 'CLsimresult.dat'
!
      integer, parameter :: n = 60                  ! input data size
      real(sp), dimension(n) :: t  = -999.0_sp      ! time
      real(sp), dimension(n) :: x  = -999.0_sp      ! data
      real(sp), dimension(n) :: s  = -999.0_sp      ! std
!
      integer :: m = -999                           ! number of extremes
      real(sp), dimension(n) :: text = -999.0_sp    ! time (extreme)
      real(sp) :: u = -999.0_sp                     ! CL test statistic
      real(sp) :: p = -999.0_sp                     ! CL P-value
!
      integer, parameter :: nsim = 2000             ! number of
                                                    !    simulations
      real(sp), dimension(n) :: xsim = -999.0_sp    ! simulated x
      real(sp), dimension(n) :: textsim = -999.0_sp ! simulated text
      integer  :: msim = -999                       ! simulated m
      real(sp) :: usim = -999.0_sp                  ! simulated u
      real(sp) :: psim = -999.0_sp                  ! simulated p
!
      real(sp), parameter :: thres = 113.0_sp       ! threshold value
      real(sp), parameter :: tobsl = 1980.417_sp    ! left boundary
      real(sp), parameter :: tobsr = 2018.917_sp    ! right boundary
!
      integer :: i=-999                             ! counter
      integer :: isim=-999                          ! counter
      integer :: j=-999                             ! counter
      real(sp) :: z1 = -999.0_sp                    ! Gaussian variate
!
! 1.  Initial stuff
!     =============
!
      print *
      print *
      print *
      print *,                                                          &
       ' CLsim: Cox--Lewis test with simulated series'
      print *,                                                          &
       ' ============================================'
      print *
      print *
      print *,                                                          &
       ' Mudelsee M (2020) Statistical Analysis of Climate Extremes.'
      print *,                                                          &
       ' Cambridge University Press.'
      print *
      print *,                                                          &
       ' http://www.climate-risk-analysis.com'
      print *
      print *
      print *
!
! 1.1 Seed random number generator
!     ============================
!
      call ranseed
!
!
! 2.  Input data
!     ==========
!
      open (unit=1, file=infile, form='formatted', status='old')
      do i=1,n
         read (1,*) t(i),x(i),s(i)
      end do
      close (unit=1, status='keep')
!
! test
!     do i=1,n
!        print *, t(i),x(i),s(i)
!     end do
!
! 3.  Threshold events
!     ================
!
      j=0
      do i=1,n
         if (x(i) .ge. thres) then
            j=j+1
            text(j)=t(i)
         end if
      end do
      m=j
!
! 4.  CL test (data)
!     ==============
!
      call test(tobsl,tobsr,m,text(1:m),u,p)
      print *, ' Result on infile_'
      print *
      print *, ' m = ',m,'        u = ',u,'        P = ',p
!
! 5.  CL test (simulations) and output
!     ================================
!
! test (zero mean, unit standard deviation, Gaussian shape passed
!
!     open (unit=1, file='testgaussian.dat', form='formatted',          &
!           status='unknown')
!     do j=1,10000
!        call gasdev(x(1))
!        print *, '   j = ',j,'            x(1) = ',x(1)
!        write (unit=1, fmt='(1x,f16.8)') x(1)
!     end do
!     close (unit=1, status='keep')
!
      open (unit=1, file=outfile, form='formatted',                     &
            status='unknown')
      write (unit=1, fmt='(a)')                                         &
       ' CLsim: Results'                           
      write (unit=1, fmt='(a)')                                         &
       ' u_simulated  P_simulated'
      write (unit=1, fmt='(a)')                                         &
       ' '
!
      call gasdev(z1)
      do isim=1,nsim
         do i=1,n
            call gasdev(z1)
            xsim(i)=x(i)+z1*s(i)
         end do
         j=0
         do i=1,n
            if (xsim(i) .ge. thres) then
               j=j+1
               textsim(j)=t(i)
            end if
         end do
         msim=j
         call test(tobsl,tobsr,msim,textsim(1:msim),usim,psim)
         write (unit=1, fmt='(2(1x,f16.8))') usim,psim
      end do
!
! 6.  Exit
!     ======
!
      close (unit=1, status='keep')
      print *
      print *
      print *
      print *, ' Simulation results written to: ',outfile
      print *
      print *
      print *
      print *, ' CLsim terminated successfully'
      print *, ' ============================='
      print *
      stop
end program CLsim
!
! ======================================================================
!
subroutine gasdev(harvest)
      use nrtype
      use nr, only : ran
      implicit none
      real(sp), intent(out) :: harvest
!     Gaussian distribution (Numerical Recipes, modified: uses ran).
      real(sp) :: rsq,v1,v2
      real(sp), save :: g
      logical, save :: gaus_stored=.false.
      integer :: idum=1
      if (gaus_stored) then
          harvest=g
          gaus_stored=.false.
      else
          do
!            call ran(v1)
!            call ran(v2)
             v1=ran(idum)
             v2=ran(idum)
             v1=2.0_sp*v1-1.0_sp
             v2=2.0_sp*v2-1.0_sp
             rsq=v1**2+v2**2
             if (rsq > 0.0 .and. rsq < 1.0) exit
          end do
          rsq=sqrt(-2.0_sp*log(rsq)/rsq)
          harvest=v1*rsq
          g=v2*rsq
          gaus_stored=.true.
      end if
end subroutine gasdev
!
! ======================================================================
!
subroutine ranseed
      use nrtype
      use nr, only: ran
      implicit none
!     Seeds random number generator.
      integer :: seed=0
      real(sp) :: dummy=-999.0_sp
      seed=-206761862
      dummy=ran(seed)
end subroutine ranseed
!
! ======================================================================
!
subroutine test(tobsl,tobsr,m,t,u,p)
      use nrtype
      use nr, only: erfcc
      implicit none
      real(sp), intent(in) :: tobsl
      real(sp), intent(in) :: tobsr
      integer, intent(in)  :: m
      real(sp), dimension(m), intent(in) :: t
      real(sp), intent(out) :: u
      real(sp), intent(out) :: p
!
!     Test of H0: "constant risk" after
!     Cox and Lewis (1966: Sect. 3.3).
!     Under H0, u is distributed as N(0; 1).
!     p is the (one-sided) probability
!     that a value > u, for u positive, or
!                  < u, for u negative,
!     results.
!
      real(sp) :: t0
      real(sp), parameter :: half=0.5_sp
      real(sp), parameter :: twelve=12.0_sp
      t0=tobsr-tobsl
      u=sum(t)/m-tobsl-half*t0
      u=u*sqrt(twelve*m)/t0
      p=half*erfcc(abs(u)/sqrt2)
end subroutine test
!
! ======================================================================
!
!
! Used Numerical Recipes files on the computer:
!
include 'erfcc.f90'
include 'ran.f90'